
package de.hsrm.ads;

import org.junit.Test;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;


public class DLinkedPQueueTest {

	private void check(DLinkedPQueue<Integer> q, int[] vals) {
		
		DLinkedPQueue<Integer>.Node n = q.head.next;
		
		// check forward links
		for (int i=0; i<vals.length; ++i) {
			assertEquals((Integer)vals[i], n.obj);
			assertEquals(vals[i], n.priority);
			n = n.next;
		}
		
		assertTrue(n == q.tail);
		n = n.prev;
		
		// check backward links
		for (int i=vals.length-1; i>=0; --i) {
			assertEquals((Integer)vals[i], n.obj);
			assertEquals(vals[i], n.priority);						
			n = n.prev;
		}
		
		assertTrue(n == q.head);		
	}

	
	
	@Test
	public void testAdd1() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		q.add(20, 20);
		q.add(30, 30);
		q.add(10, 10);
		check(q, new int[] {30,20,10});
	}

	@Test
	public void testAdd2() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		q.add(10, 10);
		check(q, new int[] {10});
	}

	@Test
	public void testAdd3() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		q.add(10, 10);
		q.add(20, 20);
		q.add(30, 30);
		q.add(10, 10);
		q.add(10, 10);
		q.add(30, 30);
		check(q, new int[] {30,30,20,10,10,10});
	}

	@Test
	public void testPop1() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		check(q, new int[] {});
		assertEquals(null, q.pop());
	}

	@Test
	public void testPop2() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		q.add(10, 10);
		q.add(20, 20);
		q.add(30, 30);
		q.add(10, 10);
		q.add(10, 10);
		q.add(30, 30);
		assertEquals(new Integer(30), q.pop());
		assertEquals(new Integer(30), q.pop());
		assertEquals(new Integer(20), q.pop());
		assertEquals(new Integer(10), q.pop());
		assertEquals(new Integer(10), q.pop());
		assertEquals(new Integer(10), q.pop());
		assertEquals(null, q.pop());
	}

	@Test
	public void testRemoveAll1() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		q.removeAll(10);
		check(q, new int[] {});
	}
	
	@Test
	public void testRemoveAll2() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		q.add(10, 10);
		q.add(20, 20);
		q.add(30, 30);
		q.add(10, 10);
		q.add(10, 10);
		q.add(30, 30);
		q.removeAll(40);
		check(q, new int[] {30,30,20,10,10,10});
		q.removeAll(30);
		check(q, new int[] {20,10,10,10});
		q.removeAll(10);
		check(q, new int[] {20});
		q.removeAll(20);
		check(q, new int[] {});
	}
	
	@Test
	public void testRemoveAll3() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		q.add(10, 10);
		q.add(20, 20);
		q.add(30, 30);
		q.add(30, 30);
		q.removeAll(10);
		check(q, new int[] {30,30,20});
	}

	@Test
	public void testRemoveAll4() {
		DLinkedPQueue<Integer> q = new DLinkedPQueue<Integer>();
		q.add(10, 10);
		q.add(20, 20);
		q.add(30, 30);
		q.add(30, 30);
		q.removeAll(30);
		check(q, new int[] {20,10});
	}

}
